@@ -3,8 +3,8 @@ package swagger
 
 import (
 	"github.com/containers/podman/v5/pkg/domain/entities"
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
+	"github.com/docker/docker/api/types/network"
 )
 
 // Details for creating a volume
@@ -32,15 +32,15 @@ type volumeCreate struct {
 
 // Network create
 // swagger:model
-type networkCreate types.NetworkCreateRequest
+type networkCreate network.CreateRequest
 
 // Network connect
 // swagger:model
-type networkConnectRequest types.NetworkConnect
+type networkConnectRequest network.ConnectOptions
 
 // Network disconnect
 // swagger:model
-type networkDisconnectRequest types.NetworkDisconnect
+type networkDisconnectRequest network.DisconnectOptions
 
 // Network connect
 // swagger:model
