@@ -5,7 +5,6 @@ import (
 	"github.com/containers/storage/pkg/stringid"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
-	. "github.com/onsi/gomega/gexec"
 )
 
 var _ = Describe("Podman run with --mac-address flag", func() {
@@ -14,7 +13,7 @@ var _ = Describe("Podman run with --mac-address flag", func() {
 		result := podmanTest.Podman([]string{"run", "--mac-address", "92:d0:c6:0a:29:34", ALPINE, "ip", "addr"})
 		result.WaitWithDefaultTimeout()
 		if isRootless() {
-			Expect(result).Should(Exit(125))
+			Expect(result).Should(ExitWithError(125, "invalid config provided: networks and static ip/mac address can only be used with Bridge mode networking"))
 		} else {
 			Expect(result).Should(ExitCleanly())
 			Expect(result.OutputToString()).To(ContainSubstring("92:d0:c6:0a:29:34"))
