@@ -7,7 +7,6 @@ import (
 	. "github.com/containers/podman/v5/test/utils"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
-	. "github.com/onsi/gomega/gexec"
 )
 
 var _ = Describe("Podman rm", func() {
@@ -29,8 +28,7 @@ var _ = Describe("Podman rm", func() {
 
 		result := podmanTest.Podman([]string{"rm", cid})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(2))
-		Expect(result.ErrorToString()).To(ContainSubstring("containers cannot be removed without force"))
+		Expect(result).Should(ExitWithError(2, "containers cannot be removed without force"))
 	})
 
 	It("podman rm created container", func() {
@@ -169,23 +167,19 @@ var _ = Describe("Podman rm", func() {
 
 		result := podmanTest.Podman([]string{"rm", "--cidfile", "foobar", "--latest"})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(125))
-		Expect(result.ErrorToString()).To(ContainSubstring("--all, --latest, and --cidfile cannot be used together"))
+		Expect(result).Should(ExitWithError(125, "--all, --latest, and --cidfile cannot be used together"))
 
 		result = podmanTest.Podman([]string{"rm", "--cidfile", "foobar", "--all"})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(125))
-		Expect(result.ErrorToString()).To(ContainSubstring("--all, --latest, and --cidfile cannot be used together"))
+		Expect(result).Should(ExitWithError(125, "--all, --latest, and --cidfile cannot be used together"))
 
 		result = podmanTest.Podman([]string{"rm", "--cidfile", "foobar", "--all", "--latest"})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(125))
-		Expect(result.ErrorToString()).To(ContainSubstring("--all, --latest, and --cidfile cannot be used together"))
+		Expect(result).Should(ExitWithError(125, "--all, --latest, and --cidfile cannot be used together"))
 
 		result = podmanTest.Podman([]string{"rm", "--latest", "--all"})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(125))
-		Expect(result.ErrorToString()).To(ContainSubstring("--all and --latest cannot be used together"))
+		Expect(result).Should(ExitWithError(125, "--all and --latest cannot be used together"))
 	})
 
 	It("podman rm --all", func() {
@@ -214,8 +208,7 @@ var _ = Describe("Podman rm", func() {
 
 		session = podmanTest.Podman([]string{"rm", "bogus", cid})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(1))
-		Expect(session.ErrorToString()).To(ContainSubstring("\"bogus\" found: no such container"))
+		Expect(session).Should(ExitWithError(1, `no container with ID or name "bogus" found: no such container`))
 		if IsRemote() {
 			Expect(session.OutputToString()).To(BeEquivalentTo(cid))
 		}
@@ -232,8 +225,7 @@ var _ = Describe("Podman rm", func() {
 	It("podman rm bogus container", func() {
 		session := podmanTest.Podman([]string{"rm", "bogus"})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(1))
-		Expect(session.ErrorToString()).To(ContainSubstring("\"bogus\" found: no such container"))
+		Expect(session).Should(ExitWithError(1, `no container with ID or name "bogus" found: no such container`))
 	})
 
 	It("podman rm bogus container and a running container", func() {
@@ -243,13 +235,11 @@ var _ = Describe("Podman rm", func() {
 
 		session = podmanTest.Podman([]string{"rm", "bogus", "test1"})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(1))
-		Expect(session.ErrorToString()).To(ContainSubstring("\"bogus\" found: no such container"))
+		Expect(session).Should(ExitWithError(1, `no container with ID or name "bogus" found: no such container`))
 
 		session = podmanTest.Podman([]string{"rm", "test1", "bogus"})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(1))
-		Expect(session.ErrorToString()).To(ContainSubstring("\"bogus\" found: no such container"))
+		Expect(session).Should(ExitWithError(1, `no container with ID or name "bogus" found: no such container`))
 	})
 
 	It("podman rm --ignore bogus container and a running container", func() {
@@ -259,8 +249,7 @@ var _ = Describe("Podman rm", func() {
 
 		session = podmanTest.Podman([]string{"rm", "--ignore", "test1", "bogus"})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(2))
-		Expect(session.ErrorToString()).To(ContainSubstring("containers cannot be removed without force"))
+		Expect(session).Should(ExitWithError(2, "containers cannot be removed without force"))
 
 		session = podmanTest.Podman([]string{"rm", "-t", "0", "--force", "--ignore", "bogus", "test1"})
 		session.WaitWithDefaultTimeout()
@@ -292,8 +281,7 @@ var _ = Describe("Podman rm", func() {
 
 		session1 = podmanTest.Podman([]string{"rm", cid1, "-f", "--filter", "status=running"})
 		session1.WaitWithDefaultTimeout()
-		Expect(session1).Should(Exit(125))
-		Expect(session1.ErrorToString()).To(ContainSubstring("--filter takes no arguments"))
+		Expect(session1).Should(ExitWithError(125, "--filter takes no arguments"))
 
 		session1 = podmanTest.Podman([]string{"rm", "-a", "-f", "--filter", fmt.Sprintf("id=%swrongid", shortCid3)})
 		session1.WaitWithDefaultTimeout()
